/*
 * "Intelligent" removing of problematic chars from file names.
 * Ensures that all filenames will be alphanumeric, only '-' is allowed.
 * Vladimir A. Smirnov (kkatarn@sleepgate.ru, smirnov@nocnt.ru), 2009
 *
 * gcc -Wall --pedantic -O2 despace.c -o despace
 *
 * BUGBUG: it should _keep_ what must be kept. Fix it.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>

#define	MAX_OUTLEN	4096

static char *strip_debris(char *in, char *outbuf)
{
	int dotcount;
	char *out = outbuf;
	for(;*in;in++)
	{
		if(isalnum(*in) || *in == '-' || *in == '_' || *in == '.')
			/* and MC under FreeBSD+X doesn't like '_' char, too */
			*out++ = (*in == '_') ? '-' : *in;
	}
	*out = 0;
	if(!*outbuf)
		return NULL;
	/*
	 * Now, there is some chars left.
	 * But if we get only one 1st dot - it's not acceptable.
	 */
	if(*outbuf != '.')
		return outbuf;
	/* count dots */
	for(dotcount = 0,out = outbuf;*out;out++)
		if(*out == '.')
			dotcount++;
	if(dotcount < 2) /* only extension left, or even one dot! */
		return NULL;
	/* here it can't be outbuf == "."; just skip the 1st dot */
	return outbuf+1;
}

int main(int argc, char *argv[])
{
	int passcount, full_len, fname_len;
	unsigned pid;
	unsigned char pid8;
	char inbuf[MAX_OUTLEN], buf[MAX_OUTLEN];
	char *ptr, *nptr;
	if(argc != 2)
		return -1;
	pid = (unsigned)getpid();
	pid8 = (unsigned char)(pid + (pid>>8) + (pid>>16) + (pid>>24));
	/*
	 * Search for file name: the path must be left intact.
	 */
 	full_len = strlen(argv[1]);
	for(nptr = argv[1] + full_len - 1;nptr != argv[1] && *nptr != '/';nptr--);
	if(nptr != argv[1])
		nptr++;
	fname_len = full_len - (nptr - argv[1]);
	/* now nptr points to 1st char of name */
	if(!fname_len)
	{
		fprintf(stderr,"Path must contain file name\n");
		return -1;
	}
	if(fname_len >= MAX_OUTLEN)
	{
		fprintf(stderr,
			"File name too long, increase MAX_OUTLEN end recompile\n");
		return -1;
	}
	strcpy(inbuf,nptr);
	for(passcount = 0;;passcount++)
	{
		if(strip_debris(inbuf,buf))
		{
			/* reconstruct full path name; it can't be longer then argv[1] */
			char *destname = (char*)alloca(full_len+1);
			strcpy(destname,argv[1]);
			strcpy(&destname[full_len-fname_len],buf);
			if(!strcmp(argv[1],destname)) /* nothing to rename */
				return 0;
			if(rename(argv[1],destname) == 0)
				return 0;
			perror("unable to rename");
			return -1;
		}
		fprintf(stderr,"no valid chars, trying %i pass of alternation: ",
			passcount);
		switch(passcount)
		{
			case 0:
				fprintf(stderr,"setting high-bit to 0...\n");
				for(ptr = inbuf;*ptr;ptr++)
					*ptr = *ptr & 0x7F;
				break;
			case 1:
				fprintf(stderr,"one bit shift...\n");
				for(ptr = inbuf;*ptr;ptr++)
					*ptr = *ptr >> 1;
				break;
			case 2:
				fprintf(stderr,"XOR with PID...\n");
				for(ptr = inbuf;*ptr;ptr++)
					*ptr = *ptr ^ pid8;
				break;
			case 3:
				fprintf(stderr,"inversed XOR with PID...\n");
				for(ptr = inbuf;*ptr;ptr++)
					*ptr = (*ptr ^ pid8) ^ 0xFF;
				break;
			default:
				fprintf(stderr,"nothing helps!\n");
				return -1;
		}
	}
	return -1; /* make GCC happy */
}
